package javacodegeeks.zepton;

import io.github.wgilreath.ZeptoN.Zep;

import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

import java.util.ArrayList;


public class ZepPTO {

    private ZepPTO() {
    }//end constructor

    private static String[] getFileList(final String[] args) {

        final ArrayList<String> fileList = new ArrayList<>();

        for (String arg : args) {
            if (!arg.startsWith("-")) {
                fileList.add(arg);
            }//end if
        }//end for

        String[] files = new String[fileList.size()];
        files = fileList.toArray(files);

        return files;

    }//end getFileList

    private static String transpileCode(final String srcCode){

        String result = srcCode;

        final String[] primTypeList = new String[]{
                "boolean", "byte", "char", "double", "float", "int", "long", "short"
        };

        final String[] objectTypeList = new String[]{
                "Boolean", "Byte", "Character", "Double", "Float", "Integer", "Long", "Short"
        };

        for(int x=0;x<primTypeList.length;x++){

            String prim = String.format("%s[?]", primTypeList[x]); //primitive type with '?' at end

            if(result.contains(primTypeList[x])){

                String upper = String.format("%SQ",      primTypeList[x]);  //primitive to uppercase
                String type  = String.format("\\b%s\\b", primTypeList[x]); //exact match of primitive type

                result  = result
                        . replaceAll(prim,  upper)
                        . replaceAll(type,  objectTypeList[x])
                        . replaceAll(upper, primTypeList[x]);

            }//end if

        }//end for

        return result;

    }//end transpileCode

    private static void transpile(String fileNameOrig) {

        //import source code from filename.orig
        String srcCode = "";
        try {
            srcCode = new String(Files.readAllBytes(Paths.get(fileNameOrig)));
        } catch (Exception ex) {
            System.out.printf("Failure reading source file: %s!%n", fileNameOrig);
            return;
        }//end try

        //transpile to ZeptoN source from extended feature syntax
        srcCode = transpileCode(srcCode);

        String path = fileNameOrig.replace(".orig", ".zep");

        try {
            Files.write(Paths.get(path), srcCode.getBytes(Charset.defaultCharset()));
          } catch (Exception ex) {
            System.out.printf("Source code write failure!%n");
            return;
        }//end try

    }//end transpile

    private static void compile(String[] args) {

        String[] fileList = getFileList(args);

        for (String fileName : fileList) {
            String fileNameOrig = fileName.replace(".zep", ".orig");

            //rename fileName.orig to fileName.zep
            Path oldFile = Paths.get(fileName);
            Path newFile = Paths.get(fileNameOrig);
            try {
                Files.move(oldFile, newFile, StandardCopyOption.REPLACE_EXISTING);
            } catch (Exception ex) {
                System.out.println("Error: Unable to rename file!");
                continue;
            }//end try

            transpile(fileNameOrig);

        }//end for

        Zep.compile(args);

        //rename fileList filename.orig to filename.zep
        for (String fileName : fileList) {

            //delete .zep files
            Path path = Paths.get(fileName);
            try {
                Files.delete(path);
            } catch (Exception ex) {
                System.out.printf("Failure deleting file path:%s!%n", path);
            }//end try

            String fileNameOrig = fileName.replace(".zep", ".orig");

            //RENAME(fileNameOrig, fileName);
            Path oldFile = Paths.get(fileNameOrig);
            Path newFile = Paths.get(fileName);
            try {
                Files.move(oldFile, newFile, StandardCopyOption.REPLACE_EXISTING);
            } catch (Exception ex) {
                System.out.printf("Error: Unable to rename file %s!%n", oldFile);
            }//end try

        }//end for

    }//end transpile

    public static void main(String[] args){

        if (args.length == 0)
            Zep.main(args);  //let Zep transpiler issue help/info and error message
        else
            ZepPTO.compile(args);

        System.exit(0);

    }//end main

}//end class ZepPTO
